@@ -32,6 +32,7 @@ import java.util.stream.StreamSupport;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.uberfire.commons.async.DescriptiveRunnable;
+import org.uberfire.ext.metadata.engine.BatchIndexListener;
 import org.uberfire.ext.metadata.engine.MetaIndexEngine;
 import org.uberfire.ext.metadata.engine.Observer;
 import org.uberfire.ext.metadata.io.IndexableIOEvent.DeletedFileEvent;
@@ -46,7 +47,6 @@ import org.uberfire.java.nio.base.WatchContext;
 import org.uberfire.java.nio.base.dotfiles.DotFileUtils;
 import org.uberfire.java.nio.file.DeleteOption;
 import org.uberfire.java.nio.file.DirectoryNotEmptyException;
-import org.uberfire.java.nio.file.DirectoryStream;
 import org.uberfire.java.nio.file.FileSystem;
 import org.uberfire.java.nio.file.FileSystemAlreadyExistsException;
 import org.uberfire.java.nio.file.FileSystemNotFoundException;
@@ -89,6 +89,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
              executorService,
              indexersFactory,
              dispatcherFactory,
+             new NOPBatchIndexListener(),
              views);
     }
 
@@ -105,6 +106,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
              executorService,
              indexersFactory,
              dispatcherFactory,
+             new NOPBatchIndexListener(),
              views);
     }
 
@@ -121,6 +123,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
              executorService,
              indexersFactory,
              dispatcherFactory,
+             new NOPBatchIndexListener(),
              views);
     }
 
@@ -139,6 +142,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
              executorService,
              indexersFactory,
              dispatcherFactory,
+             new NOPBatchIndexListener(),
              views);
     }
 
@@ -148,6 +152,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
                                 final ExecutorService executorService,
                                 final IndexersFactory indexersFactory,
                                 final IndexerDispatcherFactory dispatcherFactory,
+                                final BatchIndexListener batchIndexListener,
                                 final Class<? extends FileAttributeView>... views) {
         super();
         this.indexEngine = checkNotNull("indexEngine",
@@ -157,7 +162,13 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         this.executorService = executorService;
         this.indexersFactory = indexersFactory;
         this.dispatcherFactory = dispatcherFactory;
-        this.batchIndex = new BatchIndex(indexEngine, observer, executorService, indexersFactory, dispatcherFactory, views);
+        this.batchIndex = new BatchIndex(indexEngine,
+                                         observer,
+                                         executorService,
+                                         indexersFactory,
+                                         dispatcherFactory,
+                                         batchIndexListener,
+                                         views);
         ensureCoreIndexerExists();
     }
 
@@ -168,6 +179,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
                                 final ExecutorService executorService,
                                 final IndexersFactory indexersFactory,
                                 final IndexerDispatcherFactory dispatcherFactory,
+                                final BatchIndexListener batchIndexListener,
                                 final Class<? extends FileAttributeView>... views) {
         super(id);
         this.indexEngine = checkNotNull("indexEngine",
@@ -176,7 +188,13 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         this.executorService = executorService;
         this.indexersFactory = indexersFactory;
         this.dispatcherFactory = dispatcherFactory;
-        this.batchIndex = new BatchIndex(indexEngine, observer, executorService, indexersFactory, dispatcherFactory, views);
+        this.batchIndex = new BatchIndex(indexEngine,
+                                         observer,
+                                         executorService,
+                                         indexersFactory,
+                                         dispatcherFactory,
+                                         batchIndexListener,
+                                         views);
         ensureCoreIndexerExists();
     }
 
@@ -187,6 +205,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
                                 final ExecutorService executorService,
                                 final IndexersFactory indexersFactory,
                                 final IndexerDispatcherFactory dispatcherFactory,
+                                final BatchIndexListener batchIndexListener,
                                 final Class<? extends FileAttributeView>... views) {
         super(watchService);
         this.indexEngine = checkNotNull("indexEngine",
@@ -196,7 +215,13 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         this.executorService = executorService;
         this.indexersFactory = indexersFactory;
         this.dispatcherFactory = dispatcherFactory;
-        this.batchIndex = new BatchIndex(indexEngine, observer, executorService, indexersFactory, dispatcherFactory, views);
+        this.batchIndex = new BatchIndex(indexEngine,
+                                         observer,
+                                         executorService,
+                                         indexersFactory,
+                                         dispatcherFactory,
+                                         batchIndexListener,
+                                         views);
         ensureCoreIndexerExists();
     }
 
@@ -208,6 +233,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
                                 final ExecutorService executorService,
                                 final IndexersFactory indexersFactory,
                                 final IndexerDispatcherFactory dispatcherFactory,
+                                final BatchIndexListener batchIndexListener,
                                 final Class<? extends FileAttributeView>... views) {
         super(id,
               watchService);
@@ -218,17 +244,24 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         this.executorService = executorService;
         this.indexersFactory = indexersFactory;
         this.dispatcherFactory = dispatcherFactory;
-        this.batchIndex = new BatchIndex(indexEngine, observer, executorService, indexersFactory, dispatcherFactory, views);
+        this.batchIndex = new BatchIndex(indexEngine,
+                                         observer,
+                                         executorService,
+                                         indexersFactory,
+                                         dispatcherFactory,
+                                         batchIndexListener,
+                                         views);
         ensureCoreIndexerExists();
     }
 
     private void ensureCoreIndexerExists() {
         boolean containsCoreIndexer = indexersFactory.getIndexers()
-                                                     .stream()
-                                                     .anyMatch(indexer -> indexer.getClass().equals(CoreIndexer.class));
+                .stream()
+                .anyMatch(indexer -> indexer.getClass().equals(CoreIndexer.class));
 
         if (!containsCoreIndexer) {
-            indexersFactory.addIndexer(new CoreIndexer(this, views));
+            indexersFactory.addIndexer(new CoreIndexer(this,
+                                                       views));
         }
     }
 
@@ -237,17 +270,11 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
             throws IllegalArgumentException, FileSystemNotFoundException,
             ProviderNotFoundException, SecurityException {
         final FileSystem fs = super.getFileSystem(uri);
-        maybeSetupBatchIndex(fs);
+        setupBatchIndex(fs);
         setupWatchService(fs);
         return fs;
     }
 
-    private void maybeSetupBatchIndex(final FileSystem fs) {
-        if (shouldPerformInitialIndex(fs)) {
-            setupBatchIndex(fs);
-        }
-    }
-
     @Override
     public FileSystem newFileSystem(final URI uri,
                                     final Map<String, ?> env)
@@ -255,26 +282,14 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
             ProviderNotFoundException, IOException, SecurityException {
         final FileSystem fs = super.newFileSystem(uri,
                                                   env);
-        maybeSetupBatchIndex(fs);
+        setupBatchIndex(fs);
         setupWatchService(fs);
         return fs;
     }
 
-    private boolean shouldPerformInitialIndex(FileSystem fs) {
-        return indexEngine.freshIndex(KObjectUtil.toKCluster(fs)) && rootDirStream(fs).filter(dir -> hasContent(dir))
-                                                                                      .findAny()
-                                                                                      .isPresent();
-    }
-
-    private boolean hasContent(Path dir) {
-        // TODO remove this filter when AF-1073 is resolved
-        try (DirectoryStream<Path> children = newDirectoryStream(dir, path -> !path.endsWith("readme.md"))) {
-            return children.iterator().hasNext();
-        }
-    }
-
     private Stream<Path> rootDirStream(FileSystem fs) {
-        return StreamSupport.stream(fs.getRootDirectories().spliterator(), false);
+        return StreamSupport.stream(fs.getRootDirectories().spliterator(),
+                                    false);
     }
 
     @Override
@@ -285,14 +300,13 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
     @Override
     public void dispose() {
         watchServicesByFS.values()
-                         .forEach(ws -> ws.close());
+                .forEach(ws -> ws.close());
         activeIndexerDispatchers.forEach(d -> d.dispose());
 
         super.dispose();
     }
 
     private void setupBatchIndex(FileSystem fs) {
-        indexEngine.prepareBatch(KObjectUtil.toKCluster(fs));
         batchIndex.runAsync(fs);
     }
 
@@ -301,7 +315,8 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
             return;
         }
         final WatchService ws = fs.newWatchService();
-        watchServicesByFS.put(fs.getName(), ws);
+        watchServicesByFS.put(fs.getName(),
+                              ws);
 
         final ExecutorService defaultInstance = this.executorService;
 
@@ -330,34 +345,48 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
 
                         @Override
                         public void run() {
-                            final KCluster kCluster = KObjectUtil.toKCluster(fs);
-                            IndexerDispatcher dispatcher = dispatcherFactory.create(indexersFactory.getIndexers(), kCluster);
-                            final Set<Path> eventRealPaths = getRealCreatedPaths(events);
-                            try {
-                                queueEvents(events, eventRealPaths, dispatcher);
-                                scheduleIndexing(dispatcher, events, kCluster);
-                            } catch (DisposedException e) {
-                                return;
-                            }
+                            fs.getRootDirectories().forEach(rootPath -> {
+                                final KCluster kCluster = KObjectUtil.toKCluster(rootPath);
+                                IndexerDispatcher dispatcher = dispatcherFactory.create(indexersFactory.getIndexers(),
+                                                                                        kCluster);
+                                final Set<Path> eventRealPaths = getRealCreatedPaths(events);
+                                try {
+                                    queueEvents(events,
+                                                eventRealPaths,
+                                                dispatcher);
+                                    scheduleIndexing(dispatcher,
+                                                     events,
+                                                     kCluster);
+                                } catch (DisposedException e) {
+                                    return;
+                                }
+                            });
                         }
 
-                        private void scheduleIndexing(IndexerDispatcher dispatcher, List<WatchEvent<?>> events, KCluster kCluster) {
+                        private void scheduleIndexing(IndexerDispatcher dispatcher,
+                                                      List<WatchEvent<?>> events,
+                                                      KCluster kCluster) {
                             activeIndexerDispatchers.add(dispatcher);
                             dispatcher.schedule(executorService)
-                                      .thenRun(() -> LOGGER.info("Completed indexing {} events in cluster [{}].", events.size(), kCluster))
-                                      .whenComplete((result, exception) -> activeIndexerDispatchers.remove(dispatcher));
+                                    .thenRun(() -> LOGGER.info("Completed indexing {} events in cluster [{}].",
+                                                               events.size(),
+                                                               kCluster))
+                                    .whenComplete((result, exception) -> activeIndexerDispatchers.remove(dispatcher));
                         }
 
                         private void queueEvents(final List<WatchEvent<?>> events,
-                                                   final Set<Path> eventRealPaths,
-                                                   final IndexerDispatcher dispatcher) throws DisposedException {
+                                                 final Set<Path> eventRealPaths,
+                                                 final IndexerDispatcher dispatcher) throws DisposedException {
                             for (WatchEvent event : events) {
                                 if (isDisposed()) {
                                     throw new DisposedException();
                                 }
                                 try {
                                     final WatchContext context = ((WatchContext) event.context());
-                                    queueEvent(eventRealPaths, event, context, dispatcher);
+                                    queueEvent(eventRealPaths,
+                                               event,
+                                               context,
+                                               dispatcher);
                                 } catch (final Exception ex) {
                                     LOGGER.error("Error during indexing. { " + event.toString() + " }",
                                                  ex);
@@ -365,71 +394,27 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
                             }
                         }
 
-                        private void queueEvent(final Set<Path> eventRealPaths, WatchEvent event,
-                                                  final WatchContext context,
-                                                  final IndexerDispatcher dispatcher) throws DisposedException {
+                        private void queueEvent(final Set<Path> eventRealPaths,
+                                                WatchEvent event,
+                                                final WatchContext context,
+                                                final IndexerDispatcher dispatcher) throws DisposedException {
                             if (event.kind() == ENTRY_MODIFY || event.kind() == ENTRY_CREATE) {
-                                queueCreationAndModificationEvent(eventRealPaths, context, dispatcher);
+                                queueCreationAndModificationEvent(eventRealPaths,
+                                                                  context,
+                                                                  dispatcher);
                             }
 
                             if (event.kind() == StandardWatchEventKind.ENTRY_RENAME) {
-                                queueRenameEvent(context, dispatcher);
+                                queueRenameEvent(context,
+                                                 dispatcher);
                             }
 
                             if (event.kind() == StandardWatchEventKind.ENTRY_DELETE) {
-                                queueDeleteEvent(event, context, dispatcher);
+                                queueDeleteEvent(context,
+                                                 dispatcher);
                             }
                         }
 
-                        private void queueDeleteEvent(WatchEvent object, final WatchContext context, final IndexerDispatcher dispatcher) throws DisposedException {
-                            final Path oldPath = context.getOldPath();
-                            dispatcher.offer(new DeletedFileEvent(oldPath));
-                        }
-
-                        private void queueRenameEvent(final WatchContext context, final IndexerDispatcher dispatcher) throws DisposedException {
-                            final Path sourcePath = context.getOldPath();
-                            final Path destinationPath = context.getPath();
-                            dispatcher.offer(new RenamedFileEvent(sourcePath, destinationPath));
-                        }
-
-                        private void queueCreationAndModificationEvent(final Set<Path> eventRealPaths,
-                                                                         final WatchContext context,
-                                                                         final IndexerDispatcher dispatcher) throws DisposedException {
-                            // If the path to be indexed is a "dot path" but does not have an associated
-                            // "real path" index the "real path" instead. This ensures when only a
-                            // "dot path" is updated the FileAttributeView(s) are re-indexed.
-                            Path path = context.getPath();
-                            if (path.getFileName().toString().startsWith(".")) {
-                                if (!IOServiceIndexedUtil.isBlackListed(path)) {
-                                    final Path realPath = DotFileUtils.undot(path);
-                                    if (!eventRealPaths.contains(realPath)) {
-                                        path = realPath;
-                                    }
-                                }
-                            }
-
-                            if (!path.getFileName().toString().startsWith(".")) {
-                                dispatcher.offer(new IndexableIOEvent.NewFileEvent(path));
-                            }
-                        }
-
-                        private Set<Path> getRealCreatedPaths(final List<WatchEvent<?>> events) {
-                            // Get a set of "real paths" to be indexed. The "dot path" associated with the "real path"
-                            // is automatically indexed because the "dot path" contains content for FileAttributeView(s)
-                            // linked to the "real path".
-                            final Set<Path> eventRealPaths = new HashSet<>();
-                            for (WatchEvent event : events) {
-                                final WatchContext context = ((WatchContext) event.context());
-                                if (event.kind() == ENTRY_MODIFY || event.kind() == ENTRY_CREATE) {
-                                    final Path path = context.getPath();
-                                    if (!path.getFileName().toString().startsWith(".")) {
-                                        eventRealPaths.add(path);
-                                    }
-                                }
-                            }
-                            return eventRealPaths;
-                        }
-
                         private boolean isDisposed() {
                             return isDisposed || ws.isClose();
                         }
@@ -443,30 +428,124 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
     @Override
     public void delete(final Path path,
                        final DeleteOption... options) throws IllegalArgumentException, NoSuchFileException, DirectoryNotEmptyException, IOException, SecurityException {
-        super.delete(path,
-                     options);
+        cleanupIfDeletedFileSystem(path);
+        cleanupIfDeletedBranch(path);
+        deleteRepositoryFiles(path,
+                              options);
+    }
+
+    void cleanupIfDeletedFileSystem(Path path) {
         if (path instanceof FSPath) {
             FileSystem fileSystem = path.getFileSystem();
             cleanupDeletedFS(fileSystem);
         }
     }
 
+    void cleanupIfDeletedBranch(Path path) {
+        if (path.equals(path.getRoot())) {
+            indexEngine.delete(KObjectUtil.toKCluster(path));
+        }
+    }
+
+    void deleteRepositoryFiles(Path path,
+                               DeleteOption[] options) {
+        super.delete(path,
+                     options);
+    }
+
+    void queueDeleteEvent(final WatchContext context,
+                                  final IndexerDispatcher dispatcher) throws DisposedException {
+        final Path oldPath = context.getOldPath();
+        // ignore delete events for dot files, because dot files are not indexed
+        if (!isIgnored(oldPath)) {
+            dispatcher.offer(new DeletedFileEvent(oldPath));
+        }
+    }
+
+    void queueRenameEvent(final WatchContext context,
+                                  final IndexerDispatcher dispatcher) throws DisposedException {
+        final Path sourcePath = context.getOldPath();
+        final Path destinationPath = context.getPath();
+
+        if (!isIgnored(destinationPath)) {
+            dispatcher.offer(new RenamedFileEvent(sourcePath,
+                    destinationPath));
+        }
+    }
+
+    void queueCreationAndModificationEvent(final Set<Path> eventRealPaths,
+                                                   final WatchContext context,
+                                                   final IndexerDispatcher dispatcher) throws DisposedException {
+        // If the path to be indexed is a "dot path" but does not have an associated
+        // "real path" index the "real path" instead. This ensures when only a
+        // "dot path" is updated the FileAttributeView(s) are re-indexed.
+        Path path = context.getPath();
+        if (isIgnored(path)) {
+            if (!IOServiceIndexedUtil.isBlackListed(path)) {
+                final Path realPath = DotFileUtils.undot(path);
+                if (!eventRealPaths.contains(realPath)) {
+                    path = realPath;
+                }
+            }
+        }
+
+        if (!isIgnored(path)) {
+            dispatcher.offer(new IndexableIOEvent.NewFileEvent(path));
+        }
+    }
+
+    protected Set<Path> getRealCreatedPaths(final List<WatchEvent<?>> events) {
+        // Get a set of "real paths" to be indexed. The "dot path" associated with the "real path"
+        // is automatically indexed because the "dot path" contains content for FileAttributeView(s)
+        // linked to the "real path".
+        final Set<Path> eventRealPaths = new HashSet<>();
+        for (WatchEvent event : events) {
+            final WatchContext context = ((WatchContext) event.context());
+            if (event.kind() == ENTRY_MODIFY || event.kind() == ENTRY_CREATE) {
+                final Path path = context.getPath();
+                if (!isIgnored(path)) {
+                    eventRealPaths.add(path);
+                }
+            }
+        }
+        return eventRealPaths;
+    }
+
+    boolean isIgnored(Path path) {
+        if (path == null || path.getFileName() == null) return true;
+        return path.getFileName().toString().startsWith(".");
+    }
+
     private void cleanupDeletedFS(FileSystem fs) {
         WatchService ws = watchServicesByFS.remove(fs.getName());
         if (ws != null && !ws.isClose()) {
             ws.close();
         }
-        indexEngine.delete(KObjectUtil.toKCluster(fs));
+        fs.getRootDirectories().forEach(rootPath -> indexEngine.delete(KObjectUtil.toKCluster(rootPath)));
+    }
+
+    private void cleanupDeletedFS(String fsName,
+                                  Iterable<Path> rootDirectories) {
+        WatchService ws = watchServicesByFS.remove(fsName);
+        if (ws != null && !ws.isClose()) {
+            ws.close();
+        }
+        rootDirectories.forEach(rootPath -> indexEngine.delete(KObjectUtil.toKCluster(rootPath)));
     }
 
     @Override
     public boolean deleteIfExists(Path path,
                                   DeleteOption... options) throws IllegalArgumentException, DirectoryNotEmptyException, IOException, SecurityException {
+
+        Iterable<Path> rootDirectories = path.getFileSystem().getRootDirectories();
+        String fsName = path.getFileSystem().getName();
+
         final boolean result = super.deleteIfExists(path,
                                                     options);
         if (result && path instanceof FSPath) {
             FileSystem fileSystem = path.getFileSystem();
-            cleanupDeletedFS(fileSystem);
+            cleanupDeletedFS(fsName,
+                             rootDirectories);
         }
         return result;
     }
@@ -495,4 +574,17 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
             //Do nothing.
         }
     }
+
+    private static class NOPBatchIndexListener implements BatchIndexListener {
+
+        @Override
+        public void notifyIndexIngStarted(KCluster kCluster, Path path) {
+            //Do nothing.
+        }
+
+        @Override
+        public void notifyIndexIngFinished(KCluster kCluster, Path path) {
+            //Do nothing.
+        }
+    }
 }
\ No newline at end of file
